## the differential function of SCAD penalty
## !!!!!!!! "w" can NOT be a vector, or matrix
scadDiff <- function(w, lambda) {
  a = 3.7
  lambda * sign(w) * ((abs(w)<=lambda) + max(a*lambda-abs(w), 0)/(a-1)/lambda * (abs(w)>lambda))
}

## "GCV" is generalized cross-validation, "M.lambda" is used in "GCV"
## see Lifeng Wang (2007 Bioinfomatics)
M.lambda <- function(lambda, xTilde, y, alpha, gamma, relInd, dg=5) {
  sD <- rep(0, length(relInd))
  for (l in 1:(length(relInd))) {
    vn = vecnorm(gamma[((relInd[l]-1)*dg+1):(relInd[l]*dg)])
    sD[l] = scadDiff(vn, lambda) / vn
  }
  xTilde%*%(solve(t(xTilde)%*%(xTilde)+alpha/2*(kronecker(diag(sD, length(sD)), diag(1, dg)))))%*%t(xTilde)
}
GCV <- function(lambda, xTilde, y, alpha, gamma, relInd, n, dg=5) {
  ((vecnorm(y-M.lambda(lambda, xTilde, y, alpha, gamma, relInd)%*%y))^2 
   / n / (1-tr(M.lambda(lambda, xTilde, y, alpha, gamma, relInd))/n)^2)
}


## group SCAD function, see Lifeng Wang (2007 Bioinfomatics)
## X[[i]]: pA1 * ni[i], transposed to ni[i] * pA1 inside the function, usually pA1 < p
## y[[i]]: vector of length ni[i]
## gammaInit: pA1 * dg
## basis[[i]]: dg * ni[i], transposed to ni[i] * dg inside the function
## ni[i]: number of obervations for subject i
## return 1: gamma: pA1 * dg, with irrelavent variables set to 0
## return 2: relInd: selected variables, be cautious here, the index are for "X" passed to function
##                   "gSCAD", not the index for original "xData"
## return 3: u.gamma: to calculate Y_i - U_i*gamma

gSCAD <- function(X, y, gammaInit, basis, n, ni) {
  pA1 <- dim(X[[1]])[1]  ## pA1: dimension of selected variables before you get into this function
  dg <- dim(basis[[1]])[1]
  
  lh = sum(ni)
  e1 = 0.001
  e2 = 0.001
  alpha = lh
  
  ## Y vector
  vecY <- NULL
  for (i in 1:n) vecY = c(vecY, y[[i]])
  vecY <- as.matrix(vecY)

  ## gamma vector
  vecGamma <- NULL
  for (l in 1:pA1) vecGamma = c(vecGamma, gammaInit[l, ])
  vecGamma <- as.matrix(vecGamma)
  
  ## tilde X = kronecker(X, B)
  XB <- matrix(0, nrow=lh, ncol=pA1*dg)
  tag = 1
  for (i in 1:n) {
    X[[i]] = t(X[[i]])
    basis[[i]] = t(basis[[i]])
    for (j in 1:(ni[i])) {
      XB[tag,] = kronecker(X[[i]][j,], basis[[i]][j,])
      tag = tag + 1
    }
  }

  ## calculate "gamma" until convergence
  gamma.scad <- vecGamma
  vecGamma <- vecGamma - 10
  flag = 0
  relInd <- seq(1:pA1)
  irrelInd <- NULL
  totalInd <- seq(1:pA1)
  pM1 <- pA1
  ite = 1
  
  while ((vecnorm(gamma.scad-vecGamma) / vecnorm(vecGamma) > 0.0001 | flag == 1) & ite <=30) {
    #print(paste("iteration #", ite, sep=""))
    #print((vecnorm(gamma.scad-vecGamma) / vecnorm(vecGamma)))
    #print(paste("flag=", flag, sep=""))
    ## if new character is dropped, update X matrix
    if (flag == 1) {
      relInd <- setdiff(totalInd, irrelInd)
      #print(relInd)
      pM1 <- length(relInd)

      XB <- matrix(0, nrow=lh, ncol=pM1*dg)
      tag = 1
      for (i in 1:n) {
        for (j in 1:(ni[i])) {
          XB[tag,] = kronecker(X[[i]][j,relInd], basis[[i]][j,])
          tag = tag + 1
        }
      }
      flag = 0
    }
    lambda <- optimize(GCV, c(0, 50), xTilde=XB, y=vecY, alpha=alpha, 
                       gamma=gamma.scad, relInd=relInd, n=n)$minimum  
    #print(paste("lambda=", lambda, sep=""))
    vecGamma <- gamma.scad
    
    ## calculate "Sigma"
    sD <- rep(0, pM1)
    for (l in 1:pM1) {
      vn = vecnorm(vecGamma[((relInd[l]-1)*dg+1):(relInd[l]*dg)])
      sD[l] = scadDiff(vn, lambda) / vn
    }
    Sigma <- kronecker(diag(sD, length(sD)), diag(1, dg))
  
    ## calculate new "gamma"
    mtx <- t(XB)%*%XB + (alpha/2)*Sigma
    if (sum(svd(mtx)$d < 10^(-10))) mtx <- mtx + diag(e2, dim(mtx)[1])
    gamma.temp <- solve(mtx) %*% t(XB) %*% vecY

    gamma.scad = as.matrix(rep(0, length(vecGamma)))
    for (l in 1:pM1) {
      if (vecnorm(gamma.temp[((l-1)*dg+1):(l*dg)]) > e1) {
        gamma.scad[((relInd[l]-1)*dg+1):(relInd[l]*dg), 1] = gamma.temp[((l-1)*dg+1):(l*dg)]
      } else {
        flag = 1
        irrelInd = c(irrelInd, relInd[l])
      }
    }
    if (all(!is.null(irrelInd))) irrelInd = sort(irrelInd)
    #print(irrelInd)
    ite = ite + 1
  }
  
  gamma = gammaInit
  for (l in 1:pA1) gamma[l,] = gamma.scad[((l-1)*dg+1):(l*dg)]
  
  UG = NULL
  for (i in 1:n) { 
    UG[[i]] = rep(0, length(y[[i]]))
    for (l in relInd) UG[[i]] = UG[[i]] + diag(X[[i]][,l], nrow=ni[i])%*%basis[[i]]%*%gamma[l,]
  }
  #print(relInd)
  list(gamma=gamma, M1=relInd, u.gamma=UG)
}
